@@ -45,10 +45,10 @@ New Config::
     # * abort: abort the push
     auto-publish = ignore
 
-    # For some large repository with few markers, the current  for obsolescence
-    # markers discovery can get in the way. You can disable it with the
-    # configuration option below. This means all pushes and pulls will
-    # re-exchange all markers every time.
+    # For some large repositories with few markers, the current method for
+    # obsolescence marker discovery can get in the way. You can disable it by
+    # setting the following configuration option to "no". Then all pushes and
+    # pulls will re-exchange all markers every time.
     evolution.obsdiscovery = yes
 
 Obsolescence Markers Discovery
@@ -100,8 +100,8 @@ implementation of some of the algorithms::
     # ensuring no large repository will get affected.
     obshashrange.max-revs = 100000 # default is None
 
-For very large repositories, it might be useful to disable obsmarkers
-discovery (Make sure you follow release announcement to know when you can turn
+For very large repositories, it might be useful to disable obsmarkers discovery
+(make sure you follow release announcements to know when you might want to turn
 it back on)::
 
     [experimental]
@@ -298,6 +298,7 @@ from mercurial import (
     node as nodemod,
     obsolete,
     pycompat,
+    registrar,
     util,
 )
 
@@ -458,7 +459,7 @@ def setupparentcommand(ui):
     pstatusopts = [o for o in statuscmd[1] if o[1] not in inapplicable]
 
     @eh.command(b'pstatus', pstatusopts,
-                **compat.helpcategorykwargs('CATEGORY_WORKING_DIRECTORY'))
+                helpcategory=registrar.command.CATEGORY_WORKING_DIRECTORY)
     def pstatus(ui, repo, *args, **kwargs):
         """show status combining committed and uncommitted changes
 
@@ -475,7 +476,7 @@ def setupparentcommand(ui):
     pdiffopts = [o for o in diffcmd[1] if o[1] not in inapplicable]
 
     @eh.command(b'pdiff', pdiffopts,
-                **compat.helpcategorykwargs('CATEGORY_WORKING_DIRECTORY'))
+                helpcategory=registrar.command.CATEGORY_WORKING_DIRECTORY)
     def pdiff(ui, repo, *args, **kwargs):
         """show diff combining committed and uncommitted changes
 
@@ -766,8 +767,9 @@ def _findprevtarget(repo, display, movebookmark=False, topic=True):
      (b'n', b'dry-run', False,
         _(b'do not perform actions, just print what would be done'))],
     b'[OPTION]...',
+    helpcategory=registrar.command.CATEGORY_WORKING_DIRECTORY,
     helpbasic=True,
-    **compat.helpcategorykwargs('CATEGORY_WORKING_DIRECTORY'))
+)
 def cmdprevious(ui, repo, **opts):
     """update to parent revision
 
@@ -781,7 +783,7 @@ def cmdprevious(ui, repo, **opts):
         wkctx = repo[None]
         wparents = wkctx.parents()
         if len(wparents) != 1:
-            raise error.Abort(_(b'merge in progress'))
+            raise compat.StateError(_(b'merge in progress'))
         if not mergeopt:
             # we only skip the check if noconflict is set
             if ui.config(b'commands', b'update.check') == b'noconflict':
@@ -827,8 +829,9 @@ def cmdprevious(ui, repo, **opts):
       _(b'do not perform actions, just print what would be done')),
      (b'', b'abort', False, _(b'abort the interrupted next'))],
     b'[OPTION]...',
+    helpcategory=registrar.command.CATEGORY_WORKING_DIRECTORY,
     helpbasic=True,
-    **compat.helpcategorykwargs('CATEGORY_WORKING_DIRECTORY'))
+)
 def cmdnext(ui, repo, **opts):
     """update to next child revision
 
@@ -845,11 +848,11 @@ def cmdnext(ui, repo, **opts):
     if abortopt:
         evolvestate = state.cmdstate(repo)
         if not evolvestate:
-            raise error.Abort(_(b'no interrupted next to abort'))
+            raise compat.StateError(_(b'no interrupted next to abort'))
 
         evolvestate.load()
         if evolvestate[b'command'] != b'next':
-            raise error.Abort(_(b'no interrupted next to abort'))
+            raise compat.StateError(_(b'no interrupted next to abort'))
 
         pctx = repo[b'.']
         compat.clean_update(pctx)
@@ -865,7 +868,7 @@ def cmdnext(ui, repo, **opts):
         wkctx = repo[None]
         wparents = wkctx.parents()
         if len(wparents) != 1:
-            raise error.Abort(_(b'merge in progress'))
+            raise compat.StateError(_(b'merge in progress'))
 
         children = [ctx for ctx in wparents[0].children() if not ctx.obsolete()]
         topic = _getcurrenttopic(repo)
@@ -1132,16 +1135,10 @@ def _setuphelp(ui):
         if entry[0] == b"evolution":
             break
     else:
-        if util.safehasattr(help, 'TOPIC_CATEGORY_CONCEPTS'):
-            help.helptable.append(([b"evolution"],
-                                   _(b"Safely Rewriting History"),
-                                   _helploader,
-                                   help.TOPIC_CATEGORY_CONCEPTS))
-        else:
-            # hg <= 4.7 (c303d65d2e34)
-            help.helptable.append(([b"evolution"],
-                                   _(b"Safely Rewriting History"),
-                                   _helploader))
+        help.helptable.append(([b"evolution"],
+                               _(b"Safely Rewriting History"),
+                               _helploader,
+                               help.TOPIC_CATEGORY_CONCEPTS))
         help.helptable.sort()
 
 evolvestateversion = 0
@@ -1169,14 +1166,17 @@ def setupevolveunfinished(ui):
         statemod.addunfinished(b'evolve', fname=b'evolvestate',
                                continueflag=True, stopflag=True,
                                statushint=_msg,
-                               abortfunc=evolvecmd.hgabortevolve)
+                               abortfunc=evolvecmd.hgabortevolve,
+                               continuefunc=evolvecmd.hgcontinueevolve)
         statemod.addunfinished(b'pick', fname=b'pickstate', continueflag=True,
-                               abortfunc=cmdrewrite.hgabortpick)
+                               abortfunc=cmdrewrite.hgabortpick,
+                               continuefunc=cmdrewrite.hgcontinuepick)
         _fixup_msg = _(b'To continue:    hg fixup --continue\n'
                        b'To abort:       hg fixup --abort\n')
         statemod.addunfinished(b'fixup', fname=b'fixup-state',
                                continueflag=True, statushint=_fixup_msg,
-                               abortfunc=cmdrewrite.hgabortfixup)
+                               abortfunc=cmdrewrite.hgabortfixup,
+                               continuefunc=cmdrewrite.hgcontinuefixup)
     else:
         # hg <= 5.0 (5f2f6912c9e6)
         estate = (b'evolvestate', False, False, _(b'evolve in progress'),
@@ -1206,5 +1206,5 @@ def setupevolveunfinished(ui):
 @eh.wrapfunction(hg, 'clean')
 def clean(orig, repo, *args, **kwargs):
     ret = orig(repo, *args, **kwargs)
-    util.unlinkpath(repo.vfs.join(b'evolvestate'), ignoremissing=True)
+    repo.vfs.unlinkpath(b'evolvestate', ignoremissing=True)
     return ret
