@@ -28,6 +28,7 @@ from mercurial import (
     patch,
     phases,
     pycompat,
+    registrar,
     scmutil,
     util,
     repair,
@@ -104,8 +105,9 @@ interactiveopt = [[b'i', b'interactive', None, _(b'use interactive mode')]]
      (b'n', b'note', b'', _(b'store a note on amend'), _(b'TEXT')),
      ] + walkopts + commitopts + commitopts2 + commitopts3 + interactiveopt,
     _(b'[OPTION]... [FILE]...'),
+    helpcategory=registrar.command.CATEGORY_COMMITTING,
     helpbasic=True,
-    **compat.helpcategorykwargs('CATEGORY_COMMITTING'))
+)
 def amend(ui, repo, *pats, **opts):
     """combine a changeset with updates and replace it with a new one
 
@@ -171,8 +173,8 @@ def amendpatch(ui, repo, *pats, **opts):
         if opts.get('note'):
             metadata[b'note'] = opts['note']
         replacements = {(old.node(),): [newnode]}
-        compat.cleanupnodes(repo, replacements, operation=b'amend',
-                            metadata=metadata)
+        scmutil.cleanupnodes(repo, replacements, operation=b'amend',
+                             metadata=metadata)
         phases.retractboundary(repo, tr, old.phase(), [newnode])
         compat.clean_update(repo[newnode])
 
@@ -351,7 +353,8 @@ def _commitfiltered(repo, ctx, match, target=None, message=None, user=None,
      (b'n', b'note', b'', _(b'store a note on uncommit'), _(b'TEXT')),
      ] + commands.walkopts + commitopts + commitopts2 + commitopts3,
     _(b'[OPTION]... [FILE]...'),
-    **compat.helpcategorykwargs('CATEGORY_CHANGE_MANAGEMENT'))
+    helpcategory=registrar.command.CATEGORY_CHANGE_MANAGEMENT,
+)
 def uncommit(ui, repo, *pats, **opts):
     """move changes from parent revision to working directory
 
@@ -464,13 +467,13 @@ def uncommit(ui, repo, *pats, **opts):
             metadata[b'note'] = opts['note']
 
         replacements = {(old.node(),): [newid]}
-        compat.cleanupnodes(repo, replacements, operation=b"uncommit",
-                            metadata=metadata)
+        scmutil.cleanupnodes(repo, replacements, operation=b"uncommit",
+                             metadata=metadata)
         phases.retractboundary(repo, tr, oldphase, [newid])
         if opts.get('revert'):
             compat.clean_update(repo[newid])
         else:
-            with repo.dirstate.parentchange(), compat.parentchange(repo):
+            with compat.changing_parents(repo):
                 compat.movedirstate(repo, repo[newid], match)
         if not repo[newid].files():
             ui.warn(_(b"new changeset is empty\n"))
@@ -587,8 +590,9 @@ def _patchtocommit(ui, repo, old, fp, message=None, extras=None):
      (b'n', b'note', b'', _(b'store a note on fold'), _(b'TEXT')),
      ] + commitopts + commitopts2 + commitopts3,
     _(b'hg fold [OPTION]... [-r] REV...'),
+    helpcategory=registrar.command.CATEGORY_CHANGE_MANAGEMENT,
     helpbasic=True,
-    **compat.helpcategorykwargs('CATEGORY_CHANGE_MANAGEMENT'))
+)
 def fold(ui, repo, *revs, **opts):
     """fold multiple revisions into a single one
 
@@ -698,8 +702,8 @@ def fold(ui, repo, *revs, **opts):
                                                         commitopts=commitopts)
             phases.retractboundary(repo, tr, targetphase, [newid])
             replacements = {tuple(ctx.node() for ctx in allctx): [newid]}
-            compat.cleanupnodes(repo, replacements, operation=b"fold",
-                                metadata=metadata)
+            scmutil.cleanupnodes(repo, replacements, operation=b"fold",
+                                 metadata=metadata)
             tr.close()
         finally:
             tr.release()
@@ -716,7 +720,8 @@ def fold(ui, repo, *revs, **opts):
      (b'n', b'note', b'', _(b'store a note on metaedit'), _(b'TEXT')),
      ] + commitopts + commitopts2 + commitopts3,
     _(b'hg metaedit [OPTION]... [[-r] REV]...'),
-    **compat.helpcategorykwargs('CATEGORY_CHANGE_MANAGEMENT'))
+    helpcategory=registrar.command.CATEGORY_CHANGE_MANAGEMENT,
+)
 def metaedit(ui, repo, *revs, **opts):
     """edit commit information
 
@@ -823,17 +828,9 @@ def metaedit(ui, repo, *revs, **opts):
                     metadata[b'note'] = opts['note']
 
                 phases.retractboundary(repo, tr, targetphase, [newid])
-                # Use this condition as a proxy since the commit we care about
-                # (b99903534e06) didn't change any signatures.
-                if util.safehasattr(scmutil, 'nullrev'):
-                    mapping = {tuple(ctx.node() for ctx in allctx): (newid,)}
-                    scmutil.cleanupnodes(repo, mapping, operation=b"metaedit",
-                                         metadata=metadata)
-                else:
-                    # hg <= 4.7 (b99903534e06)
-                    mapping = {ctx.node(): (newid,) for ctx in allctx}
-                    scmutil.cleanupnodes(repo, mapping, operation=b"metaedit",
-                                         metadata=metadata)
+                replacements = {tuple(ctx.node() for ctx in allctx): (newid,)}
+                scmutil.cleanupnodes(repo, replacements, operation=b"metaedit",
+                                     metadata=metadata)
             else:
                 ui.status(_(b"nothing changed\n"))
             tr.close()
@@ -879,8 +876,9 @@ def _getmetadata(**opts):
      (b'B', b'bookmark', [], _(b"remove revs only reachable from given"
                                b" bookmark"), _(b'BOOKMARK'))] + metadataopts,
     _(b'[OPTION]... [-r] REV...'),
+    helpcategory=registrar.command.CATEGORY_CHANGE_MANAGEMENT,
     helpbasic=True,
-    **compat.helpcategorykwargs('CATEGORY_CHANGE_MANAGEMENT'))
+)
 # XXX -U  --noupdate option to prevent wc update and or bookmarks update ?
 def cmdprune(ui, repo, *revs, **opts):
     """mark changesets as obsolete or succeeded by another changeset
@@ -922,10 +920,8 @@ def cmdprune(ui, repo, *revs, **opts):
     fold = opts.get('fold')
     split = opts.get('split')
 
-    options = [o for o in (r'pair', r'fold', r'split') if opts.get(o)]
-    if 1 < len(options):
-        _opts = pycompat.sysbytes(r', '.join(options))
-        raise error.Abort(_(b"can only specify one of %s") % _opts)
+    compat.check_at_most_one_arg(opts, 'pair', 'fold', 'split')
+    compat.check_at_most_one_arg(opts, 'biject', 'fold', 'split')
 
     if bookmarks:
         reachablefrombookmark = rewriteutil.reachablefrombookmark
@@ -935,13 +931,13 @@ def cmdprune(ui, repo, *revs, **opts):
             rewriteutil.deletebookmark(repo, repomarks, bookmarks)
 
     if not revs:
-        raise error.Abort(_(b'no revisions specified to prune'))
+        raise compat.InputError(_(b'no revisions specified to prune'))
 
     wlock = lock = tr = None
     try:
         wlock = repo.wlock()
         lock = repo.lock()
-        rewriteutil.precheck(repo, revs, b'prune')
+        rewriteutil.precheck(repo, revs, b'prune', check_divergence=bool(succs))
         tr = repo.transaction(b'prune')
         # defines pruned changesets
         precs = []
@@ -957,20 +953,20 @@ def cmdprune(ui, repo, *revs, **opts):
         sucs.sort()
         sucs = tuple(repo[n] for n in sucs)
         if not biject and len(sucs) > 1 and len(precs) > 1:
-            msg = b"Can't use multiple successors for multiple precursors"
+            msg = b"cannot use multiple successors for multiple precursors"
             hint = _(b"use --pair to mark a series as a replacement"
                      b" for another")
-            raise error.Abort(msg, hint=hint)
+            raise compat.InputError(msg, hint=hint)
         elif biject and len(sucs) != len(precs):
-            msg = b"Can't use %d successors for %d precursors"\
+            msg = b"cannot use %d successors for %d precursors"\
                 % (len(sucs), len(precs))
-            raise error.Abort(msg)
+            raise compat.InputError(msg)
         elif (len(precs) == 1 and len(sucs) > 1) and not split:
             msg = b"please add --split if you want to do a split"
-            raise error.Abort(msg)
+            raise compat.InputError(msg)
         elif len(sucs) == 1 and len(precs) > 1 and not fold:
             msg = b"please add --fold if you want to do a fold"
-            raise error.Abort(msg)
+            raise compat.InputError(msg)
         elif biject:
             replacements = {(p.node(),): [s.node()] for p, s in zip(precs, sucs)}
         else:
@@ -1059,8 +1055,8 @@ def cmdprune(ui, repo, *revs, **opts):
                     break
         if len(sucs) == 1 and len(precs) > 1 and fold:
             replacements = {tuple(p.node() for p in precs): [s.node() for s in sucs]}
-        compat.cleanupnodes(repo, replacements, operation=b"prune", moves=moves,
-                            metadata=metadata)
+        scmutil.cleanupnodes(repo, replacements, operation=b"prune", moves=moves,
+                             metadata=metadata)
 
         # informs that changeset have been pruned
         ui.status(_(b'%i changesets pruned\n') % len(precs))
@@ -1076,8 +1072,9 @@ def cmdprune(ui, repo, *revs, **opts):
      (b'n', b'note', b'', _(b"store a note on split"), _(b'TEXT')),
      ] + commitopts + commitopts2 + commitopts3,
     _(b'hg split [OPTION]... [-r REV] [FILE]...'),
+    helpcategory=registrar.command.CATEGORY_CHANGE_MANAGEMENT,
     helpbasic=True,
-    **compat.helpcategorykwargs('CATEGORY_CHANGE_MANAGEMENT'))
+)
 def cmdsplit(ui, repo, *pats, **opts):
     """split a changeset into smaller changesets
 
@@ -1260,7 +1257,8 @@ def cmdsplit(ui, repo, *pats, **opts):
       b'divergence')],
     # allow to choose the seed ?
     _(b'[OPTION]... [-r] REV...'),
-    **compat.helpcategorykwargs('CATEGORY_CHANGE_MANAGEMENT'))
+    helpcategory=registrar.command.CATEGORY_CHANGE_MANAGEMENT,
+)
 def touch(ui, repo, *revs, **opts):
     """create successors identical to their predecessors but the changeset ID
 
@@ -1331,7 +1329,7 @@ def touchnodes(ui, repo, revs, displayer, **opts):
         tr = repo.currenttransaction()
         phases.retractboundary(repo, tr, ctx.phase(), [new])
         if ctx in repo[None].parents():
-            with repo.dirstate.parentchange(), compat.parentchange(repo):
+            with compat.changing_parents(repo):
                 repo.dirstate.setparents(new, node.nullid)
 
 @eh.command(
@@ -1341,7 +1339,8 @@ def touchnodes(ui, repo, revs, displayer, **opts):
      (b'a', b'abort', False, b'abort interrupted pick'),
      ] + mergetoolopts,
     _(b'[OPTION]... [-r] REV'),
-    **compat.helpcategorykwargs('CATEGORY_CHANGE_MANAGEMENT'))
+    helpcategory=registrar.command.CATEGORY_CHANGE_MANAGEMENT,
+)
 def cmdpick(ui, repo, *revs, **opts):
     """move a commit onto the working directory parent and update to it.
 
@@ -1367,7 +1366,7 @@ def cmdpick(ui, repo, *revs, **opts):
                 raise error.Abort(_(b"cannot specify both --continue and "
                                     b"revision"))
             if not pickstate:
-                raise error.Abort(_(b"no interrupted pick state exists"))
+                raise compat.StateError(_(b"no interrupted pick state exists"))
 
             pickstate.load()
             orignode = pickstate[b'orignode']
@@ -1403,33 +1402,46 @@ def cmdpick(ui, repo, *revs, **opts):
                 pickstate.save()
                 raise error.InterventionRequired(_(b"unresolved merge conflicts"
                                                    b" (see hg help resolve)"))
-        overrides = {(b'phases', b'new-commit'): origctx.phase()}
-        with repo.ui.configoverride(overrides, b'pick'):
-            newnode = repo.commit(text=origctx.description(),
-                                  user=origctx.user(),
-                                  date=origctx.date(), extra=origctx.extra())
-        repo.dirstate.setbranch(origctx.branch())
+        return _dopick(ui, repo, pickstate, origctx)
 
-        if pickstate:
-            pickstate.delete()
-        if newnode is None:
-            replacements = {(origctx.node(),): []}
-        else:
-            newctx = repo[newnode]
-            replacements = {(origctx.node(),): [newctx.node()]}
-        compat.cleanupnodes(repo, replacements, operation=b"pick")
+def _dopick(ui, repo, pickstate, origctx):
+    """shared logic for performing or continuing a pick"""
+    overrides = {(b'phases', b'new-commit'): origctx.phase()}
+    with repo.ui.configoverride(overrides, b'pick'):
+        newnode = repo.commit(text=origctx.description(),
+                              user=origctx.user(),
+                              date=origctx.date(), extra=origctx.extra())
+    repo.dirstate.setbranch(origctx.branch())
 
-        if newnode is None:
-            ui.warn(_(b"note: picking %d:%s created no changes to commit\n") %
-                    (origctx.rev(), origctx))
-            return 0
+    if pickstate:
+        pickstate.delete()
+    if newnode is None:
+        replacements = {(origctx.node(),): []}
+    else:
+        newctx = repo[newnode]
+        replacements = {(origctx.node(),): [newctx.node()]}
+    scmutil.cleanupnodes(repo, replacements, operation=b"pick")
 
+    if newnode is None:
+        ui.warn(_(b"note: picking %d:%s created no changes to commit\n") %
+                (origctx.rev(), origctx))
         return 0
 
+    return 0
+
+def hgcontinuepick(ui, repo):
+    """logic to continue pick using 'hg continue'"""
+    with repo.wlock(), repo.lock():
+        pickstate = state.cmdstate(repo, path=b'pickstate')
+        pickstate.load()
+        orignode = pickstate[b'orignode']
+        origctx = repo[orignode]
+        return _dopick(ui, repo, pickstate, origctx)
+
 def abortpick(ui, repo, pickstate, abortcmd=False):
     """logic to abort pick"""
     if not pickstate and not abortcmd:
-        raise error.Abort(_(b"no interrupted pick state exists"))
+        raise compat.StateError(_(b"no interrupted pick state exists"))
     pickstate.load()
     pctxnode = pickstate[b'oldpctx']
     compat.clean_update(repo[pctxnode])
@@ -1453,8 +1465,8 @@ def hgabortpick(ui, repo):
         (b'', b'abort', False, _(b'abort an interrupted fixup')),
     ],
     _(b'[OPTION]... [-r] REV'),
+    helpcategory=registrar.command.CATEGORY_COMMITTING,
     helpbasic=True,
-    **compat.helpcategorykwargs('CATEGORY_COMMITTING')
 )
 def fixup(ui, repo, node=None, **opts):
     """add working directory changes to an arbitrary revision
@@ -1485,31 +1497,34 @@ def _perform_fixup(ui, repo, node, **opts):
     abortopt = opts.get('abort')
     if node or opts.get('rev'):
         if contopt:
-            raise error.Abort(_(b'cannot specify a revision with --continue'))
+            raise compat.InputError(_(b'cannot specify a revision with --continue'))
         if abortopt:
-            raise error.Abort(_(b'cannot specify a revision with --abort'))
+            raise compat.InputError(_(b'cannot specify a revision with --abort'))
     # state file for --continue/--abort cases
     fixup_state = state.cmdstate(repo, b'fixup-state')
     if contopt:
         if not fixup_state.exists():
-            raise error.Abort(_(b'no interrupted fixup to continue'))
+            raise compat.StateError(_(b'no interrupted fixup to continue'))
         fixup_state.load()
         return continuefixup(ui, repo, fixup_state)
     if abortopt:
         if not fixup_state.exists():
-            raise error.Abort(_(b'no interrupted fixup to abort'))
+            raise compat.StateError(_(b'no interrupted fixup to abort'))
         fixup_state.load()
         return abortfixup(ui, repo, fixup_state)
 
     if node and opts.get('rev'):
-        raise error.Abort(_(b'please specify just one revision'))
+        raise compat.InputError(_(b'please specify just one revision'))
     if not node:
         node = opts.get('rev')
     if not node:
-        raise error.Abort(_(b'please specify a revision to fixup'))
+        raise compat.InputError(_(b'please specify a revision to fixup'))
     target_ctx = scmutil.revsingle(repo, node)
 
-    fixup_state[b'startnode'] = repo[b'.'].node()
+    fixup_state.addopts({
+        b'bookmarkchanges': [],
+        b'startnode': repo[b'.'].node(),
+    })
 
     tr = repo.transaction(b'fixup')
     with util.acceptintervention(tr):
@@ -1543,7 +1558,7 @@ def _perform_fixup(ui, repo, node, **opts):
         )
         phases.retractboundary(repo, tr, target_ctx.phase(), [newid])
         replacements = {tuple(ctx.node() for ctx in allctx): [newid]}
-        compat.cleanupnodes(repo, replacements, operation=b'fixup')
+        scmutil.cleanupnodes(repo, replacements, operation=b'fixup')
         fixup_state.delete()
         compat.update(repo.unfiltered()[tempnode])
         return 0
@@ -1571,11 +1586,18 @@ def continuefixup(ui, repo, fixup_state):
         )
         phases.retractboundary(repo, tr, target_ctx.phase(), [newid])
         replacements = {tuple(ctx.node() for ctx in allctx): [newid]}
-        compat.cleanupnodes(repo, replacements, operation=b'fixup')
+        scmutil.cleanupnodes(repo, replacements, operation=b'fixup')
         fixup_state.delete()
         compat.update(repo.unfiltered()[tempnode])
         return 0
 
+def hgcontinuefixup(ui, repo):
+    """logic for handling `hg continue' for fixup"""
+    with repo.wlock(), repo.lock():
+        fixup_state = state.cmdstate(repo, b'fixup-state')
+        fixup_state.load()
+        return continuefixup(ui, repo, fixup_state)
+
 def abortfixup(ui, repo, fixup_state):
     """logic for handling of `hg fixup --abort`"""
     with repo.wlock(), repo.lock():
@@ -1594,3 +1616,10 @@ def abortfixup(ui, repo, fixup_state):
     ui.status(_(b'working directory is now at %s\n') % pctx)
     fixup_state.delete()
     return 0
+
+def hgabortfixup(ui, repo):
+    """logic to abort fixup using 'hg abort'"""
+    with repo.wlock(), repo.lock():
+        fixup_state = state.cmdstate(repo, path=b'fixup-state')
+        fixup_state.load()
+        return abortfixup(ui, repo, fixup_state)
