@@ -8,21 +8,21 @@ Compatibility module
 
 import contextlib
 
+from mercurial.i18n import _
 from mercurial import (
     cmdutil,
     context,
     copies as copiesmod,
     dirstate,
+    error,
     hg,
     logcmdutil,
     merge as mergemod,
     node,
     obsolete,
-    registrar,
-    repair,
+    pycompat,
     scmutil,
     util,
-    ui as uimod,
 )
 
 # hg <= 5.2 (c21aca51b392)
@@ -53,26 +53,6 @@ TROUBLES = {
     r'PHASEDIVERGENT': b'phase-divergent',
 }
 
-# hg <= 4.6 (bec1212eceaa)
-if util.safehasattr(uimod.ui, 'makeprogress'):
-    def progress(ui, topic, pos, item=b"", unit=b"", total=None):
-        progress = ui.makeprogress(topic, unit, total)
-        if pos is not None:
-            progress.update(pos, item=item)
-        else:
-            progress.complete()
-else:
-    def progress(ui, topic, pos, item=b"", unit=b"", total=None):
-        ui.progress(topic, pos, item, unit, total)
-
-# XXX: Better detection of property cache
-if r'predecessors' not in dir(obsolete.obsstore):
-    @property
-    def predecessors(self):
-        return self.precursors
-
-    obsolete.obsstore.predecessors = predecessors
-
 def memfilectx(repo, ctx, fctx, flags, copied, path):
     # XXX Would it be better at the module level?
     varnames = context.memfilectx.__init__.__code__.co_varnames  # pytype: disable=attribute-error
@@ -90,19 +70,13 @@ def memfilectx(repo, ctx, fctx, flags, copied, path):
                                   copied=copied.get(path))  # pytype: disable=wrong-keyword-args
     return mctx
 
-try:
-    # hg <= 4.6 (46c2b19a1263)
-    bmrevset = repair.stripbmrevset  # pytype: disable=module-attr
-except AttributeError:
-    bmrevset = scmutil.bookmarkrevs
-
 hg48 = util.safehasattr(copiesmod, 'stringutil')
 # code imported from Mercurial core at ae17555ef93f + patch
 def fixedcopytracing(repo, c1, c2, base):
     """A complete copy-patse of copies._fullcopytrace with a one line fix to
     handle when the base is not parent of both c1 and c2. This should be
     converted in a compat function once https://phab.mercurial-scm.org/D3896
-    gets in and once we drop support for 4.6, this should be removed."""
+    gets in and once we drop support for 4.9, this should be removed."""
 
     from mercurial import pathutil
     copies = copiesmod
@@ -121,12 +95,8 @@ def fixedcopytracing(repo, c1, c2, base):
     # an endpoint is "dirty" if it isn't a descendant of the merge base
     # if we have a dirty endpoint, we need to trigger graft logic, and also
     # keep track of which endpoint is dirty
-    if util.safehasattr(base, 'isancestorof'):
-        dirtyc1 = not base.isancestorof(_c1)
-        dirtyc2 = not base.isancestorof(_c2)
-    else: # hg <= 4.6 (fbec9c0b32d3)
-        dirtyc1 = not base.descendant(_c1)
-        dirtyc2 = not base.descendant(_c2)
+    dirtyc1 = not base.isancestorof(_c1)
+    dirtyc2 = not base.isancestorof(_c2)
     graft = dirtyc1 or dirtyc2
     tca = base
     if graft:
@@ -358,15 +328,6 @@ fixupstreamed = util.safehasattr(scmutil, 'movedirstate')
 if not fixupstreamed:
     copiesmod._fullcopytracing = fixedcopytracing
 
-# help category compatibility
-# hg <= 4.7 (c303d65d2e34)
-def helpcategorykwargs(categoryname):
-    """Backwards-compatible specification of the helpategory argument."""
-    category = getattr(registrar.command, categoryname, None)
-    if not category:
-        return {}
-    return {'helpcategory': category}
-
 # nodemap.get and index.[has_node|rev|get_rev]
 # hg <= 5.2 (02802fa87b74)
 def getgetrev(cl):
@@ -376,9 +337,15 @@ def getgetrev(cl):
     return cl.nodemap.get
 
 @contextlib.contextmanager
-def parentchange(repo):
+def changing_parents(repo):
+    if util.safehasattr(repo.dirstate, 'changing_parents'):
+        changing_parents = repo.dirstate.changing_parents(repo)
+    else:
+        # hg <= 6.3 (7a8bfc05b691)
+        changing_parents = repo.dirstate.parentchange()
     try:
-        yield
+        with changing_parents:
+            yield
     finally:
         # hg <= 5.2 (85c4cd73996b)
         if util.safehasattr(repo, '_quick_access_changeid_invalidate'):
@@ -408,21 +375,6 @@ else:
     def clean_update(ctx):
         hg.updaterepo(ctx.repo(), ctx.node(), overwrite=True)
 
-def cleanupnodes(repo, replacements, operation, moves=None, metadata=None):
-    # Use this condition as a proxy since the commit we care about
-    # (b99903534e06) didn't change any signatures.
-    if util.safehasattr(scmutil, 'nullrev'):
-        fixedreplacements = replacements
-    else:
-        # hg <= 4.7 (b99903534e06)
-        fixedreplacements = {}
-        for oldnodes, newnodes in replacements.items():
-            for oldnode in oldnodes:
-                fixedreplacements[oldnode] = newnodes
-
-    scmutil.cleanupnodes(repo, replacements=fixedreplacements, operation=operation,
-                         moves=moves, metadata=metadata)
-
 if util.safehasattr(cmdutil, 'format_changeset_summary'):
     def format_changeset_summary_fn(ui, repo, command, default_spec):
         def show(ctx):
@@ -436,6 +388,53 @@ else:
         return logcmdutil.changesetdisplayer(ui, repo,
                                              {b'template': default_spec}).show
 
+if util.safehasattr(cmdutil, 'check_at_most_one_arg'):
+    def check_at_most_one_arg(opts, *args):
+        return cmdutil.check_at_most_one_arg(opts, *args)
+else:
+    # hg <= 5.2 (d587937600be)
+    def check_at_most_one_arg(opts, *args):
+        def to_display(name):
+            return pycompat.sysbytes(name).replace(b'_', b'-')
+
+        previous = None
+        for x in args:
+            if opts.get(x):
+                if previous:
+                    raise InputError(_(b'cannot specify both --%s and --%s')
+                                     % (to_display(previous), to_display(x)))
+                previous = x
+        return previous
+
+if util.safehasattr(cmdutil, 'check_incompatible_arguments'):
+    code = cmdutil.check_incompatible_arguments.__code__
+    if r'others' in code.co_varnames[:code.co_argcount]:
+        def check_incompatible_arguments(opts, first, others):
+            return cmdutil.check_incompatible_arguments(opts, first, others)
+    else:
+        # hg <= 5.3 (d4c1501225c4)
+        def check_incompatible_arguments(opts, first, others):
+            return cmdutil.check_incompatible_arguments(opts, first, *others)
+else:
+    # hg <= 5.2 (023ad45e2fd2)
+    def check_incompatible_arguments(opts, first, others):
+        for other in others:
+            check_at_most_one_arg(opts, first, other)
+
+# allowdivergenceopt is a much newer addition to obsolete.py
+# hg <= 5.8 (ba6881c6a178)
+allowdivergenceopt = b'allowdivergence'
+def isenabled(repo, option):
+    if option == allowdivergenceopt:
+        if obsolete._getoptionvalue(repo, obsolete.createmarkersopt):
+            return obsolete._getoptionvalue(repo, allowdivergenceopt)
+        else:
+            # note that we're not raising error.Abort when divergence is
+            # allowed, but creating markers is not, even on older hg versions
+            return False
+    else:
+        return obsolete.isenabled(repo, option)
+
 if util.safehasattr(dirstate.dirstate, 'set_clean'):
     movedirstate = scmutil.movedirstate
 else:  # hg <= 5.8 (8a50fb0784a9)
@@ -529,3 +528,25 @@ if not markdirtyfixed:
         }
 
     context.overlayworkingctx._markdirty = fixedmarkdirty
+
+if util.safehasattr(dirstate.dirstate, 'get_entry'):
+    def dirchanges(dirstate):
+        return [
+            f for f in dirstate if not dirstate.get_entry(f).maybe_clean
+        ]
+else:
+    # hg <= 5.9 (dcd97b082b3b)
+    def dirchanges(dirstate):
+        return [f for f in dirstate if dirstate[f] != b'n']
+
+if util.safehasattr(error, 'InputError'):
+    InputError = error.InputError
+else:
+    # hg <= 5.6 (8d72e29ad1e0)
+    InputError = error.Abort
+
+if util.safehasattr(error, 'StateError'):
+    StateError = error.StateError
+else:
+    # hg <= 5.6 (527ce85c2e60)
+    StateError = error.Abort
