@@ -26,6 +26,7 @@ from mercurial import (
     obsutil,
     phases,
     pycompat,
+    registrar,
     repair,
     scmutil,
     simplemerge,
@@ -133,7 +134,7 @@ def _solveorphan(ui, repo, orig, evolvestate, display, dryrun=False,
             ui.write_err(msg)
             return (False, b".")
         if exc.splitflag:
-            splitsucc = utility.select_split_successor(ui, repo, obs)
+            splitsucc = utility.select_split_successor(ui, repo, repo[exc.rev])
             if not splitsucc:
                 msg = _(b"could not solve instability, "
                         b"ambiguous destination: "
@@ -281,7 +282,7 @@ def _resolvephasedivergent(ui, repo, prec, bumped):
                                flag=obsolete.bumpedfix, operation=b'evolve')
     bmupdate(newid)
     # reroute the working copy parent to the new changeset
-    with repo.dirstate.parentchange(), compat.parentchange(repo):
+    with compat.changing_parents(repo):
         repo.dirstate.setparents(newid, nodemod.nullid)
     return (True, replacementnode)
 
@@ -614,7 +615,7 @@ def _completecontentdivergent(ui, repo, progresscb, divergent, other,
         publicdiv = repo[publicnode]
         otherdiv = other if other.mutable() else divergent
 
-        with repo.dirstate.parentchange(), compat.parentchange(repo):
+        with compat.changing_parents(repo):
             compat.movedirstate(repo, repo[publicnode])
         # check if node to be committed has changes same as public one
         s = publicdiv.status()
@@ -627,7 +628,7 @@ def _completecontentdivergent(ui, repo, progresscb, divergent, other,
             compat.mergestate.clean(repo)
             return (True, publicnode)
 
-    with repo.dirstate.parentchange(), compat.parentchange(repo):
+    with compat.changing_parents(repo):
         compat.movedirstate(repo, repo[resparent])
 
     # merge the branches
@@ -757,11 +758,7 @@ def bypassphase(repo, relation, flag=0, metadata=None, operation=b'evolve'):
         if saveeffectflag:
             # The effect flag is saved in a versioned field name for
             # future evolution
-            try:
-                effectflag = obsutil.geteffectflag(prec, (succ,))
-            except TypeError:
-                # hg <= 4.7 (bae6f1418a95)
-                effectflag = obsutil.geteffectflag((prec, (succ,)))
+            effectflag = obsutil.geteffectflag(prec, (succ,))
             metadata[obsutil.EFFECTFLAGFIELD] = b"%d" % effectflag
 
         # create markers
@@ -962,10 +959,18 @@ def _relocate(repo, orig, dest, evolvestate, pctx=None, keepbranch=False,
     _finalizerelocate(repo, orig, dest, nodenew, tr, category, evolvestate)
     return nodenew
 
+# This is copied from hgext.rebase._savegraft()
+def _savegraft(ctx, extra):
+    s = ctx.extra().get(b'source', None)
+    if s is not None:
+        extra[b'source'] = s
+    s = ctx.extra().get(b'intermediate-source', None)
+    if s is not None:
+        extra[b'intermediate-source'] = s
+
 def _relocatecommit(repo, orig, dest, pctx, keepbranch, commitmsg, update):
-    extra = dict(orig.extra())
-    if b'branch' in extra:
-        del extra[b'branch']
+    extra = {}
+    _savegraft(orig, extra)
     extra[b'rebase_source'] = orig.hex()
     targetphase = max(orig.phase(), phases.draft)
     configoverrides = {
@@ -1001,8 +1006,7 @@ def _relocatecommitondisk(repo, orig, dest, pctx, keepbranch, commitmsg, extra):
         if not orig.topic() and repo.vfs.exists(b'topic'):
             repo.vfs.unlink(b'topic')
         else:
-            with repo.vfs.open(b'topic', b'w') as f:
-                f.write(orig.topic())
+            repo.vfs.write(b'topic', orig.topic())
 
     stats = merge.graft(repo, orig, pctx, [b'destination', b'evolving'], True)
 
@@ -1539,8 +1543,8 @@ def divergentsets(repo, ctx):
                               b' in the repo')),
      ] + mergetoolopts,
     _(b'[OPTIONS]...'),
+    helpcategory=registrar.command.CATEGORY_CHANGE_MANAGEMENT,
     helpbasic=True,
-    **compat.helpcategorykwargs('CATEGORY_CHANGE_MANAGEMENT')
 )
 def evolve(ui, repo, **opts):
     """solve troubled changesets in your repository
@@ -1678,7 +1682,7 @@ def _performevolve(ui, repo, **opts):
     # Continuation handling
     if contopt:
         if not evolvestate:
-            raise error.Abort(_(b'no interrupted evolve to continue'))
+            raise compat.StateError(_(b'no interrupted evolve to continue'))
         evolvestate.load()
         headnode = continueevolve(ui, repo, evolvestate)
         if evolvestate[b'command'] != b'evolve':
@@ -1690,7 +1694,7 @@ def _performevolve(ui, repo, **opts):
         evolvestate.delete()
     elif stopopt:
         if not evolvestate:
-            raise error.Abort(_(b'no interrupted evolve to stop'))
+            raise compat.StateError(_(b'no interrupted evolve to stop'))
         evolvestate.load()
         stopevolve(ui, repo, evolvestate)
         if evolvestate[b'command'] != b'evolve':
@@ -1700,7 +1704,7 @@ def _performevolve(ui, repo, **opts):
         evolvestate.delete()
     elif abortopt:
         if not evolvestate:
-            raise error.Abort(_(b'no interrupted evolve to abort'))
+            raise compat.StateError(_(b'no interrupted evolve to abort'))
         evolvestate.load()
         # `hg next --evolve` in play
         if evolvestate[b'command'] != b'evolve':
@@ -1768,7 +1772,8 @@ def _performevolve(ui, repo, **opts):
         if showprogress:
             progress.complete()
 
-    _cleanup(ui, repo, startnode, shouldupdate, headnode)
+    if not dryrunopt:
+        _cleanup(ui, repo, startnode, shouldupdate, headnode)
 
 def _solveonerev(ui, repo, rev, evolvestate, activetopic, dryrunopt, confirmopt,
                  progresscb, targetcat):
@@ -1822,7 +1827,7 @@ def solveobswdp(ui, repo, opts):
         ctx = repo[utility._singlesuccessor(repo, oldctx)]
     except utility.MultipleSuccessorsError as exc:
         if exc.splitflag:
-            splitsucc = utility.select_split_successor(ui, repo, oldctx)
+            splitsucc = utility.select_split_successor(ui, repo, repo[exc.rev])
             if splitsucc:
                 ctx = repo[splitsucc]
             else:
@@ -2047,6 +2052,23 @@ def continueevolve(ui, repo, evolvestate):
     progress.complete()
     return headnode
 
+def hgcontinueevolve(ui, repo):
+    """logic for continuing evolve using 'hg continue'"""
+    with repo.wlock(), repo.lock():
+        evolvestate = state.cmdstate(repo)
+        evolvestate.load()
+        headnode = continueevolve(ui, repo, evolvestate)
+        if evolvestate[b'command'] != b'evolve':
+            evolvestate.delete()
+            return
+        startnode = evolvestate[b'startnode']
+        shouldupdate = False
+        if b'update' in evolvestate:
+            shouldupdate = evolvestate[b'update']
+        evolvestate.delete()
+
+        _cleanup(ui, repo, startnode, shouldupdate, headnode)
+
 def _continuecontentdivergent(ui, repo, evolvestate, progresscb):
     """function to continue the interrupted content-divergence resolution."""
     tr = repo.transaction(b'evolve')
@@ -2177,13 +2199,11 @@ def _completerelocation(ui, repo, evolvestate):
 
     orig = repo[evolvestate[b'current']]
     ctx = orig
-    source = ctx.extra().get(b'source')
+
     extra = {}
-    if source:
-        extra[b'source'] = source
-        extra[b'intermediate-source'] = ctx.hex()
-    else:
-        extra[b'source'] = ctx.hex()
+    _savegraft(ctx, extra)
+    extra[b'rebase_source'] = orig.hex()
+
     user = ctx.user()
     date = ctx.date()
     message = ctx.description()
@@ -2203,14 +2223,14 @@ def _completerelocation(ui, repo, evolvestate):
             # p1 is obsolete and p2 is not obsolete, current working
             # directory parent should be successor of p1, so we should
             # set dirstate parents to (succ of p1, p2)
-            with repo.dirstate.parentchange(), compat.parentchange(repo):
+            with compat.changing_parents(repo):
                 repo.dirstate.setparents(currentp1,
                                          ctxparents[1].node())
         elif p2obs and not p1obs:
             # p2 is obsolete and p1 is not obsolete, current working
             # directory parent should be successor of p2, so we should
             # set dirstate parents to (succ of p2, p1)
-            with repo.dirstate.parentchange(), compat.parentchange(repo):
+            with compat.changing_parents(repo):
                 repo.dirstate.setparents(ctxparents[0].node(),
                                          currentp1)
 
@@ -2218,12 +2238,12 @@ def _completerelocation(ui, repo, evolvestate):
             # both the parents were obsoleted, if orphanmerge is set, we
             # are processing the second parent first (to keep parent order)
             if evolvestate.get(b'orphanmerge'):
-                with repo.dirstate.parentchange(), compat.parentchange(repo):
+                with compat.changing_parents(repo):
                     repo.dirstate.setparents(ctxparents[0].node(),
                                              currentp1)
             pass
     else:
-        with repo.dirstate.parentchange(), compat.parentchange(repo):
+        with compat.changing_parents(repo):
             repo.dirstate.setparents(repo.dirstate.p1(), nodemod.nullid)
 
     with repo.ui.configoverride(overrides, b'evolve-continue'):
