@@ -19,6 +19,7 @@ from mercurial import (
     obsutil,
     patch,
     pycompat,
+    registrar,
     scmutil,
     util,
 )
@@ -28,7 +29,6 @@ from mercurial.utils import dateutil
 from mercurial.i18n import _
 
 from . import (
-    compat,
     exthelper,
 )
 
@@ -62,7 +62,7 @@ def addtouchnoise(ui):
      (b'o', b'origin', True, _(b'show origin of changesets instead of fate')),
      ] + commands.formatteropts,
     _(b'hg obslog [OPTION]... [[-r] REV]...'),
-    **compat.helpcategorykwargs('CATEGORY_CHANGE_NAVIGATION'))
+    helpcategory=registrar.command.CATEGORY_CHANGE_NAVIGATION)
 def cmdobshistory(ui, repo, *revs, **opts):
     """show the obsolescence history of the specified revisions
 
@@ -265,11 +265,7 @@ class obsmarker_printer(logcmdutil.changesetprinter):
 
         super(obsmarker_printer, self).__init__(ui, repo, *args, **kwargs)
         diffopts = kwargs.get('diffopts', {})
-
-        # hg <= 4.6 (3fe1c9263024)
-        if not util.safehasattr(self, "_includediff"):
-            self._includediff = diffopts and diffopts.get(b'patch')
-
+        self._includediff = diffopts and diffopts.get(b'patch')
         self.template = diffopts and diffopts.get(b'template')
         self.filter = diffopts and diffopts.get(b'filternonlocal')
         self.origin = diffopts and diffopts.get(b'origin')
@@ -384,18 +380,19 @@ class missingchangectx(object):
         # If we don't have it locally, it's obsolete
         return True
 
-def cyclic(graph):
-    """Return True if the directed graph has a cycle.
-    The graph must be represented as a dictionary mapping vertices to
-    iterables of neighbouring vertices. For example:
+def findcycle(graph):
+    """Return a set of vertices that form a cycle in the directed graph.
 
-    >>> cyclic({1: (2,), 2: (3,), 3: (1,)})
-    True
-    >>> cyclic({1: (2,), 2: (3,), 3: (4,)})
-    False
+    The first detected cycle is returned. If there's no cycle, this function
+    return None. The graph must be represented as a dictionary mapping vertices
+    to iterables of neighbouring vertices. For example:
+
+    >>> sorted(findcycle({1: (2,), 2: (3,), 3: (1,)}))
+    [1, 2, 3]
+    >>> print(findcycle({1: (2,), 2: (3,), 3: (4,)}))
+    None
 
     Taken from: https://codereview.stackexchange.com/a/86067
-
     """
     visited = set()
     o = object()
@@ -416,7 +413,7 @@ def cyclic(graph):
         else:
             path_set.remove(path.pop())
             stack.pop()
-    return False
+    return None
 
 def _obshistorywalker(repo, revs, walksuccessors=False, filternonlocal=False):
     """ Directly inspired by graphmod.dagwalker,
@@ -445,7 +442,7 @@ def _obshistorywalker(repo, revs, walksuccessors=False, filternonlocal=False):
 
         # If we likely have a cycle
         if not validcandidates:
-            cycle = cyclic(nodesucc)
+            cycle = findcycle(nodesucc)
             assert cycle
 
             # Then choose a random node from the cycle
