@@ -54,43 +54,52 @@ def _formatrevs(repo, revs, maxrevs=4):
         summary %= (node.short(tonode(first)), numrevs - 1)
     return summary
 
-def precheck(repo, revs, action=b'rewrite'):
+def precheck(repo, revs, action=b'rewrite', check_divergence=True):
     """check if <revs> can be rewritten
 
     <action> can be used to control the commit message.
     """
-    # If this attribute exists, then core's rewriteutil is recent enough
-    # that it has all the features from our own implementation.
-    if util.safehasattr(corerewriteutil, 'find_new_divergence_from'):
-        return corerewriteutil.precheck(repo, revs, action)
+    # hg <= 6.1 (d4752aeb20f1)
+    code = corerewriteutil.precheck.__code__
+    if r'check_divergence' in code.co_varnames[:code.co_argcount]:
+        return corerewriteutil.precheck(repo, revs, action,
+                                        check_divergence=check_divergence)
 
     # hg <= 5.8 (d90f6237)
     if node.nullrev in revs:
         msg = _(b"cannot %s the null revision") % (action)
         hint = _(b"no changeset checked out")
-        raise error.Abort(msg, hint=hint)
+        raise compat.InputError(msg, hint=hint)
     if any(util.safehasattr(r, 'rev') for r in revs):
         msg = b"rewriteutil.precheck called with ctx not revs"
         repo.ui.develwarn(msg)
         revs = (r.rev() for r in revs)
+    if len(repo[None].parents()) > 1:
+        raise compat.StateError(_(b"cannot %s changesets while merging")
+                                % action)
     publicrevs = repo.revs(b'%ld and public()', revs)
     if publicrevs:
         summary = _formatrevs(repo, publicrevs)
         msg = _(b"cannot %s public changesets: %s") % (action, summary)
         hint = _(b"see 'hg help phases' for details")
-        raise error.Abort(msg, hint=hint)
+        raise compat.InputError(msg, hint=hint)
     newunstable = disallowednewunstable(repo, revs)
     if newunstable:
-        msg = _(b"%s will orphan %i descendants")
+        msg = _(b"cannot %s changeset, as that will orphan %i descendants")
         msg %= (action, len(newunstable))
         hint = _(b"see 'hg help evolution.instability'")
-        raise error.Abort(msg, hint=hint)
-    divrisk = revs_hascontentdivrisk(repo, revs)
+        raise compat.InputError(msg, hint=hint)
+    if not check_divergence:
+        return
     allowdivergence = compat.isenabled(repo, compat.allowdivergenceopt)
-    if divrisk and not allowdivergence:
-        localdiv = repo[divrisk[0]]
-        otherdiv, base = repo[divrisk[1][0]], repo[divrisk[1][1]]
-        msg = _(b"%s of %s creates content-divergence "
+    if allowdivergence:
+        return
+    divergence = new_divergence(repo, revs)
+    if divergence:
+        local, other, common_prec = divergence
+        localdiv = repo[local]
+        otherdiv, base = repo[other], repo[common_prec]
+        msg = _(b"cannot %s %s, as that creates content-divergence "
                 b"with %s") % (action, localdiv, otherdiv)
         if localdiv.rev() != base.rev():
             msg += _(b', from %s') % base
@@ -98,17 +107,17 @@ def precheck(repo, revs, action=b'rewrite'):
                  b"'hg help evolution.instability'")
         if repo.ui.verbose:
             if localdiv.rev() != base.rev():
-                msg += _(b'\n    changeset %s is an evolution of '
+                msg += _(b'\n    changeset %s is a successor of '
                          b'changeset %s') % (localdiv, base)
-            msg += _(b'\n    changeset %s already have a successors as '
+            msg += _(b'\n    changeset %s already has a successor in '
                      b'changeset %s\n'
                      b'    rewriting changeset %s would create '
                      b'"content-divergence"\n'
                      b'    set experimental.evolution.allowdivergence=True to '
-                     b'overwrite this check') % (base, otherdiv, localdiv)
+                     b'skip this check') % (base, otherdiv, localdiv)
             hint = _(b"see 'hg help evolution.instability' for details "
                      b"on content-divergence")
-        raise error.Abort(msg, hint=hint)
+        raise compat.InputError(msg, hint=hint)
 
 def bookmarksupdater(repo, oldid, tr):
     """Return a callable update(newid) updating the current bookmark
@@ -121,13 +130,16 @@ def bookmarksupdater(repo, oldid, tr):
             repo._bookmarks.applychanges(repo, tr, bmchanges)
     return updatebookmarks
 
-def revs_hascontentdivrisk(repo, revs):
+def new_divergence(repo, revs):
+    """Return a tuple (rev, divergent, common_precursor) if rewriting one of
+    the revs will create content-divergence.
+    """
     obsrevs = repo.revs(b'%ld and obsolete()', revs)
     for r in obsrevs:
-        div = precheck_contentdiv(repo, repo[r])
-        if div:
-            return [r, div]
-    return []
+        div_prec = precheck_contentdiv(repo, repo[r])
+        if div_prec:
+            divergent, common_prec = div_prec
+            return (r, divergent, common_prec)
 
 def disallowednewunstable(repo, revs):
     """Check that editing <revs> will not create disallowed unstable
@@ -270,6 +282,7 @@ def rewrite(repo, old, head, newbases, commitopts):
         if commitopts.get(b'edit'):
             editor = cmdutil.commitforceeditor
         new = wctx.tomemctx(text=message,
+                            branch=extra[b'branch'],
                             parents=newbases,
                             date=date,
                             extra=extra,
